* DB_EXCL.PRG		(c) 1992 - 2002, Peisch Custom Software, Inc.
* www.peisch.com
parameters DBName, Alias        && DB name wanted and alias to use (if any)
*   Opens a database file Exclusive, whether already in use or not.
*   Returns .T. if successful, otherwise .F.
*   Displays window message for user while waiting, allows Escape.
* USAGE:
*    if not DB_EXCL('ITEMS')           && then don't try to pack it
* Note: DBName will be the alias when the file is used, unless an alias is
*       specified.
* Problem: A file lock is not sufficient for exclusive, because someone
*   could be using the file without a record lock.  We have to use the
*   error mechanism to tell whether we can do it.
*      ----------------------------
* 05/21/92 - Added alias parameter-BP
* 04/17/92 - Save and restore settings of Error and Escape
* 03/31/92 - Original version by D.Covill

private OldErr, OldEsc, OldSet, Err, Result, Esc, Cnt

do case
case not empty(Alias) and used(Alias)
   select (Alias)
case used(DBName)
   select (DBName)
endcase
OldErr = on('error')
OldEsc = on('escape')
OldSet = set('escape')
store .F. to Esc, Result
on error Err = .T.         && trap the "not available" error
Cnt = 1
do while .T.
   Err = .F.
   * don't waste time with the window unless we need a third try (the first might just
   * be to remove the index flag and the second will open the table)
   if empty(Alias)
      if not used(DBName)
         select 0
      endif
      use (DBName) exclusive
      * Turn off the Err flag and try again in case the error was "Structural CDX
      * not found".  Doing the USE again will fix this.
      Err = .F.
      use (DBName) exclusive
   else
      if not used(Alias)
         select 0
      endif
      use (DBName) alias (Alias) exclusive
      * Turn off the Err flag and try again in case the error was "Structural CDX
      * not found".  Doing the USE again will fix this.
      Err = .F.
      use (DBName) alias (Alias) exclusive
   endif
   if not Err
      m.Result = .T.
      exit               && it worked
   endif
   if Cnt > 2
      * set up the window and prepare to wait a while
      wait window nowait "Waiting for Exclusive Use of "+DBName+" - press Esc to quit"
      set escape on
      on escape Esc = .T.           && detect the escape key
      if Esc
         exit               && user pressed Escape
      endif
      wait "" timeout 1     && wait 1 second
   endif
   Cnt = Cnt+1
enddo
on error &OldErr         && restore old settings
on escape &OldEsc
set escape &OldSet
wait clear

return m.Result
*--------------------------------------------------------------------
